import znc
import threading

class autodeop(znc.Module):
    description = "Auto-deop after 10 seconds in #re-opless channel"
    module_types = [znc.CModInfo.NetworkModule]
    
    def __init__(self):
        super().__init__()
        self.timers = {}
        
    def OnLoad(self, args, message):
        """Called when module is loaded"""
        return True
    
    def OnRaw(self, line):
        """Intercept all raw messages from server"""
        line_str = str(line).strip()
        
        # Check for MODE in #re-opless
        if " MODE #re-opless " in line_str:
            # Parse MODE message
            # Format: :nick!user@host MODE #re-opless +o target
            try:
                parts = line_str.split()
                if len(parts) >= 5:
                    # parts[0] = :nick!user@host
                    # parts[1] = MODE
                    # parts[2] = #re-opless
                    # parts[3] = +o (or other modes)
                    # parts[4] = target nick
                    
                    if parts[1] == "MODE" and parts[2].lower() == "#re-opless":
                        modes = parts[3]
                        
                        # If there's +o in modes
                        if "+o" in modes:
                            mynick = self.GetNetwork().GetNick()
                            target = parts[4]
                            
                            if target.lower() == mynick.lower():
                                # Schedule deop
                                self.ScheduleDeop("#re-opless")
                                
            except Exception:
                pass
                                    
        return znc.CONTINUE
    
    def OnJoin(self, nick, channel):
        """Called when someone joins the channel"""
        # If we join the channel
        if nick.GetNick() == self.GetNetwork().GetNick():
            if channel.GetName().lower() == "#re-opless":
                # Check if we already have OP after 2 seconds
                threading.Timer(2.0, self.CheckInitialOp, args=[channel.GetName()]).start()
                    
        return znc.CONTINUE
    
    def CheckInitialOp(self, channel_name):
        """Check if we have OP when joining channel"""
        channel = self.GetNetwork().FindChan(channel_name)
        if channel:
            mynick = self.GetNetwork().GetNick()
            user = channel.FindNick(mynick)
            if user and user.HasPerm('@'):
                self.ScheduleDeop(channel_name)
    
    def ScheduleDeop(self, channel_name):
        """Schedule OP removal after 10 seconds"""
        # Cancel existing timer if present
        if channel_name in self.timers:
            self.timers[channel_name].cancel()
            
        # Create new timer
        timer = threading.Timer(10.0, self.RemoveOp, args=[channel_name])
        self.timers[channel_name] = timer
        timer.start()
    
    def RemoveOp(self, channel_name):
        """Remove OP from specified channel"""
        mynick = self.GetNetwork().GetNick()
        
        # Send command to remove OP
        cmd = f"MODE {channel_name} -o {mynick}"
        self.PutIRC(cmd)
        
        # Remove timer from list
        if channel_name in self.timers:
            del self.timers[channel_name]
    
    def OnShutdown(self):
        """Called when module is unloaded"""
        # Cancel all active timers
        for timer in self.timers.values():
            timer.cancel()
        self.timers.clear()
